#!/bin/bash
#
#
# This script will fix path for the Nagios command pipe and
# the RT CLI in the appropriate scripts
#
#
# RT/Nagios gateway - Request Tracker and Nagios bi-directional gateway
# Copyright (C) 2010  Cedval Info Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jean-Philippe Houde <jhoude@cedvalinfo.com>
#
# Modification history: 17-03-2010-01 - Original script
#


# Asking for the Nagios plugin directory
until [ "$PLUGINDIR" ]; do
	read -p "Nagios plugin directory (location of submit_*_rt.sh): " PLUGINDIR

	if [ ! -d $PLUGINDIR ]; then
		echo "ERROR: The plugin directory $PLUGINDIR doesn't exist" 
		echo ""
		unset PLUGINDIR
	fi
done


# Asking for the Nagios command pipe
until [ "$NAGPIPE" ]; do
	read -p "Nagios command pipe path: " NAGPIPE

	if [ ! -p $NAGPIPE ]; then
		echo "ERROR: Command pipe not found in $NAGPIPE"
		echo ""
		unset NAGPIPE
	fi
done


# Asking for alert queue(s)
COUNT=0
while [ 1 ]; do
	read -p "Enter Nagios alert queue name (one mandatory, one queue per line): " QUEUE_[$COUNT]

	if [ "$COUNT" = "0" -a "${QUEUE_[$COUNT]}" = "" ]; then
		echo "ERROR: You need at least one queue (single queue mode)"
		echo ""
	elif [ "${QUEUE_[$COUNT]}" = "" ]; then
		break
	else
		COUNT=$((COUNT + 1))
	fi
done


# Displaying the summary
echo ""
echo "Summary:"
echo ""
echo "Nagios plugin directory: $PLUGINDIR"
echo "Nagios command pipe: $NAGPIPE"
echo "Queue(s):"

COUNT=0
until [ "${QUEUE_[$COUNT]}" = "" ]; do
        echo "  - ${QUEUE_[$COUNT]}"

        COUNT=$((COUNT + 1))
done

echo ""

read -p "The above configuration is OK? (Y/n) " CONFIRM

if [ "$CONFIRM" = "n" -o "$CONFIRM" = "N" ]; then
	echo "No confirmation, aborting..."
	exit 1
else
	echo "Generating configuration..."
fi

# Creating the Nagios configuration from the template
DIRNAME=`dirname $0`

# Creating the RT scrips from the template
echo -n "RT scrips... "

ESCNAGPIPE=`echo $NAGPIPE | sed s/"\/"/"\\\\\\\\\/"/g`

COUNT=0
until [ "${QUEUE_[$COUNT]}" = "" ]; do 
	echo -n "Processing queue ${QUEUE_[$COUNT]}... "

	cat $DIRNAME/templates/RT-SCRIPS.txt | sed s/___QUEUE___/"${QUEUE_[$COUNT]}"/g | sed s/___NAGPIPE___/"$ESCNAGPIPE"/g >> $DIRNAME/RT-SCRIPS.txt

	if [ "$?" = "0" ]; then
		echo "Done"
	else
		echo "Failed"
	fi

	COUNT=$((COUNT + 1))
done

echo -n "Nagios configuration... "

ESCPLUGINDIR=`echo $PLUGINDIR | sed s/"\/"/"\\\\\\\\\/"/g`

if [ "${QUEUE_[1]}" = "" ]; then
	cat $DIRNAME/templates/NAGIOS-CONFIG-SINGLE.txt | sed s/___QUEUE___/"${QUEUE_[1]}"/g | sed s/___PLUGINDIR___/"$ESCPLUGINDIR"/g > $DIRNAME/NAGIOS-CONFIG.txt

	if [ "$?" = "0" ]; then
		NAGCONFRET=0
	else
		NAGCONFRET=1
	fi
else
	cat $DIRNAME/templates/NAGIOS-CONFIG-MULTI-1.txt | sed s/___QUEUE___/"${QUEUE_[1]}"/g | sed s/___PLUGINDIR___/"$ESCPLUGINDIR"/g > $DIRNAME/NAGIOS-CONFIG.txt

	if [ "$?" = "0" ]; then
		NAGCONFRET=0
	else
		NAGCONFRET=1
	fi

	COUNT=0
	until [ "${QUEUE_[$COUNT]}" = "" ]; do 
		cat $DIRNAME/templates/NAGIOS-CONFIG-MULTI-2.txt | sed s/___COUNT___/"$COUNT"/g | sed s/___QUEUE___/"${QUEUE_[1]}"/g >> $DIRNAME/NAGIOS-CONFIG.txt

		if [ "$NAGCONFRET" = "0" ]; then
			if [ "$?" = "0" ]; then
				NAGCONFRET=0
			else
				NAGCONFRET=1
			fi
		fi

		COUNT=$((COUNT + 1))
	done
fi

if [ "$NAGCONFRET" = "0" ]; then
	echo "Done"
else
	echo "Failed"
fi

