#!/bin/sh
#
#
# Submit Nagios HOST notifications to RT through the rt command
#
#
# RT/Nagios gateway - Request Tracker and Nagios bi-directional gateway
# Copyright (C) 2010  Cedval Info Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jean-Philippe Houde <jhoude@cedvalinfo.com>
#
# Modification history: 15-05-2007-01 - Original script
#			03-12-2009-01 - Added queue support, JP Houde
#


# Nagios variables from the command line
HOST="$1"
HOSTADDRESS="$2"
HOSTSTATE="$3"
NOTIFICATIONTYPE="$4"
NOTIFICATIONNUMBER="$5"
HOSTOUTPUT="$6"
HOSTACKAUTHOR="$7"
HOSTACKCOMMENT="$8"
LONGDATETIME="$9"
shift 9
QUEUE="$1"

# Debug info to the log (0 = no, 1 = yes)
DEBUG=0
# The Nagios command file
NAGIOSCMD="$OPTDIR/var/rw/nagios.cmd"
# The URL for RT, WITHOUT the tailing slash
RTURL="/rt"
# The RT command
RTCMD=/opt/rt3/bin/rt
# Automatically resolve the ticket
# no: Don't close the ticket at any time
# onrecovery: Close the ticket on recovery
RESOLVE="no"
# Log errors there
LOG="/usr/local/nagios/var/rt/submit_host_rt.log"	

# This variable will be the comment Nagios will add to the ticket
# It is spead on many lines as it will appear in RT
RTCOMMENT="***** Nagios Alert *****

Notification Type: $NOTIFICATIONTYPE
Host: $HOST
Address: $HOSTADDRESS
State: $HOSTSTATE
Date/Time: $LONGDATETIME
Info: $HOSTOUTPUT"

#
## Nothing to edit beyond this point
#

if [ ! -d "`dirname $LOG`" ]; then
	mkdir -p `dirname $LOG`
fi

echo "`basename $0` $@" >> $LOG

# We make sure we have all the needed arguments
if [ "$#" != "9" ]; then
	echo "Syntax: `basename $0` [hostname] [hostaddress] [host state] [notif type] [notif number] [host output] [host ack author] [host ack comment] [long date time] [queue]" >> $LOG
	exit 3
fi


# We make sure we have the RT command
if [ ! -x $RTCMD ]; then
	echo "You need the rt command to create and manage tickets" >> $LOG
	exit 3
fi


# The host is down or unreachable
if [ "$NOTIFICATIONTYPE" = "PROBLEM" ]; then
	if [ "$NOTIFICATIONNUMBER" = "1" ]; then
		# This is the first alert for this host
		# Create the ticket here and add a comment in Nagios
		ID=`$RTCMD create -t ticket set subject="Alert - Host $HOST" priority=1 queue="$QUEUE" CF-hostname="$HOST" CF-Current_state="$HOSTSTATE" text="$RTCOMMENT" | awk '{print $3}'`

		if [ ! "$ID" = "No matching results." ]; then
			NOW=`date +%s`
			/usr/bin/printf "[%lu] ADD_HOST_COMMENT;$HOST;1;Nagios;Ticket <a href=\"$RTURL/Ticket/Display.html?id=$ID\">#$ID</a> created\n" $NOW > $NAGIOSCMD

		else
			TS=`date +"%d-%m-%Y %H:%M:%S"`
			echo "[$TS] ERROR: Unable to get ID of created ticket" >> $LOG
		fi
	else
		# This is not the first alert for this host

		# Increasing priority
		ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - Host $HOST\" AND (Status = 'new' OR Status = 'open')" | tail -1 | cut -d: -f1`

		if [ ! "$ID" = "No matching results." ]; then
			PRIO=`$RTCMD show -t ticket -f priority $ID | grep Priority | awk '{print $2}'`
			PRIO=$((PRIO + 1))
			$RTCMD edit -t ticket set priority=$PRIO $ID

			# Get the Current_state value
			CURRSTATE=`$RTCMD show -t ticket $ID -f CF-Current_state | grep "CF-Current_state" | tail -1 | awk '{print $2}'`

			# Add a comment because the state changed
			if [ "$CURRSTATE" != "$HOSTSTATE" ]; then
				$RTCMD edit -t ticket set CF-Current_state="$HOSTSTATE" $ID
				$RTCMD comment -m "$RTCOMMENT" $ID
			fi
		else
			TS=`date +"%d-%m-%Y %H:%M:%S"`
			echo "[$TS] ERROR: Unable to get ID to increase priority" >> $LOG
		fi
	fi

elif [ "$NOTIFICATIONTYPE" = "RECOVERY" ]; then
        # Update ticket for recovery
	ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - Host $HOST\" AND (Status = 'new' OR Status = 'open')" | tail -1 | cut -d: -f1`

	if [ ! "$ID" = "No matching results." ]; then
		$RTCMD edit -t ticket set CF-Current_state="$HOSTSTATE" $ID
		$RTCMD comment -m "$RTCOMMENT" $ID

		if [ "$RESOLVE" = "onrecovery" ]; then
			$RTCMD edit -t ticket set status=resolved $ID
			NOW=`date +%s`
			/usr/bin/printf "[%lu] ADD_HOST_COMMENT;$HOST;1;Nagios;Ticket <a href=\"$RTURL/Ticket/Display.html?id=$ID\">#$ID</a> has been automatically resolved\n" $NOW > $NAGIOSCMD
		fi
	else
		TS=`date +"%d-%m-%Y %H:%M:%S"`
		echo "[$TS] ERROR: Unable to get ID to add RECOVERY comment" >> $LOG
	fi

elif [ "$NOTIFICATIONTYPE" = "ACKNOWLEDGEMENT" ]; then
	# Some stuff here
	ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - Host $HOST\" AND (Status = 'new' OR Status = 'open')" | tail -1 | cut -d: -f1`

	if [ ! "$ID" = "No matching results." ]; then
		$RTCMD edit -t ticket set owner="$HOSTACKAUTHOR" $ID
		$RTCMD comment -m "$HOSTACKCOMMENT ($HOSTACKAUTHOR)" $ID
	else
		TS=`date +"%d-%m-%Y %H:%M:%S"`
		echo "[$TS] ERROR: to get ID to set owner and add acknowledgement comment" >> $LOG
	fi
fi
