#!/bin/sh
#
#
# Submit Nagios SERVICE notifications to RT through the rt command
#
#
# RT/Nagios gateway - Request Tracker and Nagios bi-directional gateway
# Copyright (C) 2010  Cedval Info Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jean-Philippe Houde <jhoude@cedvalinfo.com>
#
# Modification history: 15-05-2007-01 - Original script
#			03-12-2009-01 - Added queue support, JP Houde
#


# Nagios variables from the command line
HOST="$1"
HOSTADDRESS="$2"
SERVICEDESC="$3"
SERVICESTATE="$4"
NOTIFICATIONTYPE="$5"
NOTIFICATIONNUMBER="$6"
SERVICEOUTPUT="$7"
SERVICEACKAUTHOR="$8"
SERVICEACKCOMMENT="$9"
shift 9
LONGDATETIME="$1"
QUEUE="$2"

# Debug output to log (0 = no, 1 = yes)
DEBUG=0
# The Nagios command file
NAGIOSCMD="/usr/local/nagios/var/rw/nagios.cmd"
# The URL for RT, WITHOUT the tailing slash
RTURL="/rt"
# The RT command
RTCMD=/opt/rt3/bin/rt
# Automatically resolve the ticket
# no: Don't close the ticket at any time
# onrecovery: Close the ticket on recovery
RESOLVE="no"
# Log errors there
LOG="/usr/local/nagios/var/rt/submit_service_rt.log"	

# This variable will be the comment Nagios will add to the ticket
# It is spead on many lines as it will appear in RT
RTCOMMENT="***** Nagios Alert *****

Notification Type: $NOTIFICATIONTYPE
Host: $HOST
Address: $HOSTADDRESS
Service: $SERVICEDESC
State: $SERVICESTATE
Date/Time: $LONGDATETIME

Additional Info:

$SERVICEOUTPUT"


#
## Nothing to edit beyond this point
#

if [ ! -d "`dirname $LOG`" ]; then
	mkdir -p `dirname $LOG`
fi

if [ "$DEBUG" = "1" ]; then echo "`basename $0` $@" >> $LOG; fi

# We make sure we have all the needed arguments
#if [ "$#" != "10" ]; then
#	echo "Syntax: `basename $0` [hostname] [hostaddress] [service desc] [service state] [notif type] [notif number] [service output] [service ack author] [service ack comment] [long date time]" >> $LOG
#	exit 3
#fi


# We make sure we have the RT command
if [ ! -x $RTCMD ]; then
	echo "You need the rt command to create and manage tickets" >> $LOG
	exit 3
fi

if [ "$DEBUG" = "1" ]; then
	echo "HOST               = $HOST" >> $LOG
	echo "HOSTADDRESS        = $HOSTADDRESS" >> $LOG
	echo "SERVICEDESC        = $SERVICEDESC" >> $LOG
	echo "SERVICESTATE       = $SERVICESTATE" >> $LOG
	echo "NOTIFICATIONTYPE   = $NOTIFICATIONTYPE" >> $LOG
	echo "NOTIFICATIONNUMBER = $NOTIFICATIONNUMBER" >> $LOG
	echo "SERVICEOUTPUT      = $SERVICEOUTPUT" >> $LOG
	echo "SERVICEACKAUTHOR   = $SERVICEACKAUTHOR" >> $LOG
	echo "SERVICEACKCOMMENT  = $SERVICEACKCOMMENT" >> $LOG
	echo "LONGDATETIME       = $LONGDATETIME" >> $LOG
	echo "Queue		 = $QUEUE" >> $LOG

	echo "Before NOTIFICATIONTYPE IF" >> $LOG
fi

# The host is down or unreachable
if [ "$NOTIFICATIONTYPE" = "PROBLEM" ]; then
	if [ "$DEBUG" = "1" ]; then echo "In IF PROBLEM" >> $LOG; fi

	if [ "$NOTIFICATIONNUMBER" = "1" ]; then
		# This is the first alert for this service
		# Create the ticket here and add a comment in Nagios
		IDLINE="$RTCMD create -t ticket set subject=\"Alert - $HOST/$SERVICEDESC\" priority=1 queue=\"$QUEUE\" text=\"Take a look at the comments for more detail\" CF-Hostname=\"$HOST\" CF-Service=\"$SERVICEDESC\" CF-Current_state=\"$SERVICESTATE\" | awk '{print \$3}'"

		echo "IDLINE = $IDLINE" >> $LOG

		ID=`$RTCMD create -t ticket set subject="Alert - $HOST/$SERVICEDESC" priority=1 queue="$QUEUE" text="$RTCOMMENT" CF-Hostname="$HOST" CF-Service="$SERVICEDESC" CF-Current_state="$SERVICESTATE" | awk '{print $3}'`
		echo "ID = $ID" >> $LOG

		if [ ! "$ID" = "No matching results." ]; then
			NOW=`date +%s`
			/usr/bin/printf "[%lu] ADD_SVC_COMMENT;$HOST;$SERVICEDESC;1;Nagios;Ticket <a href=\"$RTURL/Ticket/Display.html?id=$ID\">#$ID</a> created\n" $NOW > $NAGIOSCMD
		else
			TS=`date +"%d-%m-%Y %H:%M:%S"`
			echo "[$TS] ERROR: Unable to get ID of created ticket" >> $LOG
		fi
	else
		# This is not the first alert for this service
		# Increasing priority
		ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - $HOST/$SERVICEDESC\" AND (Status = 'new' OR Status = 'open')" | tail -1 | cut -d: -f1`

		if [ ! "$ID" = "No matching results." ]; then
			PRIO=`$RTCMD show -t ticket -f priority $ID | grep Priority | awk '{print $2}'`
			PRIO=$((PRIO + 1))
			$RTCMD edit -t ticket set priority=$PRIO $ID

			# Get the Current_state value
			CURRSTATE=`$RTCMD show -t ticket $ID -f CF-Current_state | grep "CF-Current_state" | tail -1 | awk '{print $2}'`

			# Add a comment because the state changed
			if [ "$CURRSTATE" != "$SERVICESTATE" ]; then
				echo "CURRSTATE = $CURRSTATE"
				echo "SERVICESTATE = $SERVICESTATE"
				$RTCMD edit -t ticket set CF-Current_state="$SERVICESTATE" $ID
				$RTCMD comment -m "$RTCOMMENT" $ID
			fi
		else
			TS=`date +"%d-%m-%Y %H:%M:%S"`
			echo "[$TS] ERROR: Unable to get ID to increase priority" >> $LOG
		fi
	fi

elif [ "$NOTIFICATIONTYPE" = "RECOVERY" ]; then
	if [ "$DEBUG" = "1" ]; then echo "In IF RECOVERY" >> $LOG; fi

        # Update ticket for recovery
	ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - $HOST/$SERVICEDESC\" AND (Status = 'new' OR Status = 'open')" | tail -1 | cut -d: -f1`

	if [ ! "$ID" = "No matching results." ]; then
		$RTCMD edit -t ticket set CF-Current_state="$SERVICESTATE" $ID
		$RTCMD comment -m "$RTCOMMENT" $ID

		if [ "$RESOLVE" = "onrecovery" ]; then
			$RTCMD edit -t ticket set status=resolved $ID
			NOW=`date +%s`
			/usr/bin/printf "[%lu] ADD_SVC_COMMENT;$HOST;$SERVICEDESC;1;Nagios;Ticket <a href=\"$RTURL/Ticket/Display.html?id=$ID\">#$ID</a> has been automatically resolved\n" $NOW > $NAGIOSCMD
		fi
	else
		TS=`date +"%d-%m-%Y %H:%M:%S"`
		echo "[$TS] ERROR: Unable to get ID to add RECOVERY comment" >> $LOG
	fi

elif [ "$NOTIFICATIONTYPE" = "ACKNOWLEDGEMENT" ]; then
	if [ "$DEBUG" = "1" ]; then echo "In IF ACKNOWLEDGEMENT" >> $LOG; fi

	# Some stuff here
	ID=`$RTCMD ls -t ticket "Subject LIKE \"Alert - $HOST/$SERVICEDESC\" AND (Status = 'new' OR Status = 'open)'" | tail -1 | cut -d: -f1`

	if [ ! "$ID" = "No matching results." ]; then
		$RTCMD edit -t ticket set owner="$SERVICEACKAUTHOR" $ID
		$RTCMD comment -m "$SERVICEACKCOMMENT ($SERVICEACKAUTHOR)" $ID
	else
		TS=`date +"%d-%m-%Y %H:%M:%S"`
		echo "[$TS] ERROR: to get ID to set owner and add acknowledgement comment" >> $LOG
	fi
fi

if [ "$DEBUG" = "1" ]; then echo "End of the script" >> $LOG; fi
